package net.devblock.mgh.doxdialer.plugin.doxdialer_swift_java;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.util.Log;

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import com.doximity.callwithdoxdialerlib.DoxDialerCaller;

import java.util.Dictionary;
import java.util.Map;

/** DoxdialerSwiftJavaPlugin */
public class DoxdialerSwiftJavaPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

  private static final String DOXIMITY_PACKAGE_NAME = "com.doximity.doximitydroid";
  private static Activity topActivity;
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "net.devblock.mgh.doxdialer.plugin");
    channel.setMethodCallHandler(new DoxdialerSwiftJavaPlugin());
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "net.devblock.mgh.doxdialer.plugin");
    channel.setMethodCallHandler(new DoxdialerSwiftJavaPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else {
      result.notImplemented();
    }

    if (call.method.equals("doxdialer")) {
      final String phoneNumber = call.argument("phoneNumber");

       PackageManager packageManager = topActivity.getPackageManager();
       Intent launchDialerIntent = packageManager.getLaunchIntentForPackage(DOXIMITY_PACKAGE_NAME);
       boolean doxDialerAppInstalled = launchDialerIntent != null;

       if (doxDialerAppInstalled) {
         launchDialerIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
         launchDialerIntent.setAction(Intent.ACTION_DIAL);
         launchDialerIntent.setData(Uri.parse("tel:" + phoneNumber));
         if (launchDialerIntent.resolveActivity(topActivity.getPackageManager()) != null) {
           topActivity.startActivity(launchDialerIntent);
         }
       } else {
         Intent playStoreIntent = new Intent(Intent.ACTION_VIEW,
                 Uri.parse("market://details?id=" + DOXIMITY_PACKAGE_NAME));
         boolean playStoreInstalled = playStoreIntent.resolveActivity(packageManager) != null;
         if (playStoreInstalled) {
           String packageName = topActivity.getPackageName();
           String appsFlyerUrl = "https://app.appsflyer.com/" + DOXIMITY_PACKAGE_NAME + "?pid=third_party_app&c="
                   + packageName;
           launchDialerIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(appsFlyerUrl));
           topActivity.startActivity(launchDialerIntent);
         } else {
           launchDialerIntent = new Intent(Intent.ACTION_VIEW,
                   Uri.parse("https://play.google.com/store/apps/details?id=" + DOXIMITY_PACKAGE_NAME));
           topActivity.startActivity(launchDialerIntent);
         }
       }

    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    topActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {
    topActivity = null;
  }
}