import Flutter
import UIKit

public class SwiftDoxdialerSwiftJavaPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "net.devblock.mgh.doxdialer.plugin", binaryMessenger: registrar.messenger())
    let instance = SwiftDoxdialerSwiftJavaPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    if call.method == "getPlatformVersion" {
        result("iOS " + UIDevice.current.systemVersion)
    } else {
        result(FlutterMethodNotImplemented)
    }
    
    if call.method == "doxdialer" {
        guard let args = call.arguments as? [String: Any], let phoneNumber = args["phoneNumber"] as? String else {
            result(FlutterError(code: "arguments", message: "Phone number has been null", details: nil))
            return
        }
        
        DispatchQueue.main.async {
            DoxDialerCaller.shared.dialPhoneNumber(phoneNumber)
        }
    } else {
        result(FlutterMethodNotImplemented)
    } 
  }
}

private class DoxDialerCaller {
    
    private let doximityScheme: String = "doximity"
    private lazy var doximitySchemeURL: URL? = {
        var components = URLComponents()
        components.scheme = self.doximityScheme
        components.host = ""
        
        return components.url
    }()
    
    private let openDoximityInAppStoreURL: URL? = {
        let appIdentifyingName: String = Bundle.main.bundleIdentifier ?? "Unknown"
        
        return URL(string: "https://app.appsflyer.com/id393642611?pid=third_party_app&c=\(appIdentifyingName)")
    }()
    
    static let shared = DoxDialerCaller()
    private init() {}
    
    // implement later
    func doximityIcon() -> UIImage? {
        return nil
    }
    
    // implement later
    func doximityIconAsTemplate() -> UIImage? {
        return nil
    }
    
    func dialPhoneNumber(_ phoneNumber: String) {
        if isDoximityInstalled() {
            var components = URLComponents()
            components.scheme = self.doximityScheme
            components.host = "doximity"
            components.path = "/dialer"
            components.queryItems = [URLQueryItem(name: "target_number", value: phoneNumber)]
            self.openURL(url: components.url)
        } else {
            self.openURL(url: openDoximityInAppStoreURL)
        }
    }
    
    private func isDoximityInstalled() -> Bool {
        guard let url = doximitySchemeURL else { return false }
        return UIApplication.shared.canOpenURL(url)
    }
    
    private func openURL(url: URL?) {
        guard let url = url else { return }
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(url, options: [:], completionHandler: nil)
        } else {
            // Fallback on earlier versions
        }
    }
}
